/* =============================================================================
 *
 * main.c
 *
 * This file contains the update_PWMs function and the LED task.
 *
 * v1.0	YFS	Initial release. 
 *
 * =============================================================================
*/

/*
* Copyright (2014), Cypress Semiconductor Corporation. All Rights Reserved.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*/

/* Include FreeRTOS APIs and defines */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"

/* Include PSoC system and component APIs and defines */
#include <project.h>

/* Include application function declarations and defines */
#include <Task_Defs.h>
#include <OS_Resources.h>

/* Define the event group for button presses */
EventGroupHandle_t Button_Events;

/* Use this macro to set the PWM period and compare */
#define SLIDER_MAX              (100)

/*
Defines to choose which PWMs to drive.

Each PWM drives one of the colors in the RGB LED. These defines are used to
select the color of the LED. RED drives PWM_1, GREEN drives PWM_2 and BLUE
drives PWM_3. WHITE is used to drive all three PWMs at the same time. The
NUM_LED_STATES is used by the code to wrap back to WHITE as the user cycles
through the colors.

*/
#define WHITE                   (0)
#define RED                     (1)
#define GREEN                   (2)
#define BLUE                    (3)
#define NUM_LED_STATES          (4)


/*
 * Function:	update_PWMs
 *
 * Modify the color and intensity of the RGB LED by varying the PWM duty cycles.
 * The primary colors, plus white, can be produced because a compare value of 0
 * turns the "other" LEDs off (PWM output always high). The WHITE value drives
 * the duty of all PWMs equally.
 *
 * Globals:		None
 *
 * Parameters:	led - the LED/PWM to drive
 *              compare - the duty cycle of the PWM
 *
 * Return:		void
 */
void update_PWMs( uint8 led, uint16 compare )
{
    /* Set the duty cycle - note that "0" with a "greater than" PWM is OFF */
    PWM_1_WriteCompare( ( led == WHITE || led == RED   ) ? compare : 0 );
    PWM_2_WriteCompare( ( led == WHITE || led == GREEN ) ? compare : 0 );
    PWM_3_WriteCompare( ( led == WHITE || led == BLUE  ) ? compare : 0 );
    
} /* update_PWMs() */


/*
 * Function:	LED_Task
 *
 * Modify the LED color and intensity based on the CapSense and SW2 input.
 *
 * Note that the LEDs are active-low and the PWM compare rule is greater-than
 * meaning that high compare values result in bright LEDs.
 *
 *              Start PWMs
 *           -> Check for a CapSense message, if so
 *          |       Modify the PWM duty-cycle (all PWMs)
 *          |   Check for a SW2 event, if so
 *          |       Clear the event
 *          |       Change the driven LED
 *           - Delay to let other tasks run
 *
 * Globals:     CapSense_Q and Button_Events
 *
 * Parameters:	pvParameters - not used
 *
 * Return:		None (never returns)
 */
void LED_Task( void *pvParameters )
{
    uint16 slider;                      /* Finger postion on slider */
    
    uint8 color = WHITE;                /* Remember the selected PWM to drive */  
    
    EventBits_t returnValue;            /* Return from event wait API */
    
    BaseType_t queueDataReceived;       /* Return from queue receive API */
       
    /* Start PWMs and set the maximum period to the maximum slider value */
    PWM_1_Start();
    PWM_1_WritePeriod( SLIDER_MAX );
    
    PWM_2_Start();
    PWM_2_WritePeriod( SLIDER_MAX );
    
    PWM_3_Start();
    PWM_3_WritePeriod( SLIDER_MAX );
   
    update_PWMs( WHITE, SLIDER_MAX );   /* Set all LEDS to full ON  */        
         
    for( ; ; )
    {
        /* Non-blocking call to get finger position, read into slider var */
        queueDataReceived = xQueueReceive(
            CapSense_Q,                 /* Queue resource */
            &slider,                    /* Pointer to the queue data */
            DO_NOT_WAIT );              /* Return immediately */
            
        if( queueDataReceived == pdTRUE )
        {
            /* If a message was received, change the duty cycles */
            update_PWMs( color, slider );
        }
        
        /* Non-blocking call to detect a button press event (color change)  */
        returnValue = xEventGroupWaitBits(
            Button_Events,              /* Event resource */
            SW2_EVENT,                  /* This bit set by button ISR */
            pdTRUE,                     /* Clear the event on exit */
            pdFALSE,                    /* Do not wait for all bits (irrelevant) */
            DO_NOT_WAIT );              /* Return immediately */
        
        if( returnValue == SW2_EVENT )
        {                  
            /* If an event has occurred, change the color */
            color++;
          
            /* Wrap back to white once all the colors have been shown */
            if( color == NUM_LED_STATES )
            {
                color = WHITE;
            }            
            
            update_PWMs( color, slider );
                           
        }
        
        /* Delay at same rate as CapSense_Task - Faster is wasteful, slower loses info */
        vTaskDelay( TASK_UPDATE_RATE );
        
    } /* for( ; ; ) */
    
} /* void LED_Task( void ) */
