/* =============================================================================
 *
 * main.c
 *
 * This file contains the main() function, which starts the OS, the call-backs
 * for vApplicationStackOverflowHook() and vApplicationMallocFailedHook()and the
 * interrupt handler for the switch on the kit (SW2).
 *
 * v1.0	YFS	Initial release. 
 *
 * =============================================================================
*/

/*
* Copyright (2014), Cypress Semiconductor Corporation. All Rights Reserved.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*/

/* Include FreeRTOS APIs and defines */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "timers.h"

/* Include PSoC system and component APIs and defines */
#include <project.h>

/* Include application function declarations and defines */
#include <Task_Defs.h>
#include <OS_Resources.h>
    
/* Declaration of NVIC base vector for FreeRTOS exception handling */
#define CORTEX_INTERRUPT_BASE          (16)

/* Declarations of the exception handlers for FreeRTOS */
extern void xPortPendSVHandler(void);
extern void xPortSysTickHandler(void);
extern void vPortSVCHandler(void);

/* Use this macro to set the PWM period and compare */
#define PWM_PERIOD                      (100)

/*
 * Function:    main
 *
 * Initialize the system, create the event group for button presses, the queue
 * for CapSense data, plus the ISR for SW2, and two tasks. Then start the OS.
 *
 * THEORY OF OPERATION
 *
 * This is a two-task system, where an LED-controlling task controls the
 * brightness and color of the LED based on messages from a CapSense-monitoring
 * task and an ISR, connected to SW2.
 *
 * The CapSense_Task simply scans the CapSense slider periodically
 * (TASK_UPDATE_RATE) and posts results to the CapSense_Q. The LED_Task
 * listens for these messages and updates the PWM duty cycle accordingly, 
 * either brightening or dimming the LED as directed by the user's finger.
 * The LED_task also checks for an event bit that gets set in the SW2 ISR.
 * When this occurs it switches the LED from white to red to green and to
 * blue (then repeats). This switching is achieved by modifying the PWM
 * compare values (duty cycle).
 *
 * Globals:     CapSense_Q and Button_Events
 *
 * Parameters:  None
 *
 * Return:      None (OS_Start API never returns)
 */
int main( void )
{
	/* Initialize the OS vectors - make it ready to create the task, below */
    
    /* Handler for Cortex Supervisor Call (SVC, formerly SWI) - address 11 */
    CyIntSetSysVector( CORTEX_INTERRUPT_BASE + SVCall_IRQn,
        (cyisraddress)vPortSVCHandler );
    
    /* Handler for Cortex PendSV Call - address 14 */
	CyIntSetSysVector( CORTEX_INTERRUPT_BASE + PendSV_IRQn,
        (cyisraddress)xPortPendSVHandler );    
    
    /* Handler for Cortex SYSTICK - address 15 */
	CyIntSetSysVector( CORTEX_INTERRUPT_BASE + SysTick_IRQn,
        (cyisraddress)xPortSysTickHandler );
    
    /* Create the queue for CapSense messages */
    CapSense_Q = xQueueCreate(
        CAPSENSE_QUEUE_SIZE,            /* Number of items */
        CAPSENSE_ITEM_SIZE );           /* Item size */
    
    Button_Events = xEventGroupCreate();
    
    /* Create CapSense task, which will post messages to the queue */
    xTaskCreate(
        CapSense_Task,                  /* Task function */
        CAPSENSE_TASK_NAME,             /* Task name (string) */
        DEFAULT_TASK_STACK_SIZE,        /* Task stack, allocated from heap */
        NO_TASK_ARG,                    /* No param passed to task function */
        HIGH_TASK_PRIORITY,             /* High priority */
        TASK_HANDLE_UNUSED );           /* Not using the task handle */

    /* Create LED task, which will control the intensity of the LEDs */
    xTaskCreate(
        LED_Task,                       /* Task function */
        LED_TASK_NAME,                  /* Task name (string) */
        DEFAULT_TASK_STACK_SIZE,        /* Task stack, allocated from heap */
        NO_TASK_ARG,                    /* No param passed to task function */
        LOW_TASK_PRIORITY,              /* Low priority */
        TASK_HANDLE_UNUSED );           /* Not using the task handle */
     	   
    /* Install the handler for the SW2 interrupt */
    SW2_Active_Int_ClearPending();    
    SW2_Active_Int_StartEx( SW2_ISR );
    SW2_Active_Int_SetPriority( configKERNEL_INTERRUPT_PRIORITY );

    /* Start multitasking (give control to FreeRTOS) - never returns */
    vTaskStartScheduler();

    return 0;
    
} /* int main() */


/*
 * Function:	SW2_ISR
 *
 * Signal a button press event to the LED_Task.
 *
 * Note: This ISR MUST run at configKERNEL_INTERRUPT_PRIORITY or above.
 *
 * Globals:		Uses LED_TCB
 *
 * Parameters:	None
 *
 * Return:		None (ISR)
 */
CY_ISR( SW2_ISR )
{   
    BaseType_t taskWoken = pdFALSE;     /* Detect if task switch required */

    /* Interrupt fires on press and release - we only care about presses */
    if( SW2_Read() == 0 )       /* 0 => pin is grounded (button pressed) */
    {
        /* Button pressed so signal the event */
        xEventGroupSetBitsFromISR(
            Button_Events,
            SW2_EVENT,
            &taskWoken );           /* Is a higher priority task awakened? */
    }
    
    SW2_Active_Int_ClearPending();          /* Prevent ever-repeating ISRs */
    
    /* If this event makes a higher priority taskready, tell the OS */
    if( taskWoken != pdFALSE )
        portYIELD_FROM_ISR( taskWoken );
    
} /* CY_ISR( SW2_ISR ) */


/*
 * Function:	vApplicationStackOverflowHook
 *
 * Called if a task exceeds its alloted stack. Flashes the BLUE LED.
 *
 * Requires configCHECK_FOR_STACK_OVERFLOW to be set in FreeRTOSConfig.h.
 *
 * Globals:		None
 *
 * Parameters:	Task handle and name
 *
 * Return:		Never returns - this is a fatal error condition
 */
void vApplicationStackOverflowHook( xTaskHandle pxTask, signed char *pcTaskName )
{
	/* The stack space has been execeeded for a task */
	taskDISABLE_INTERRUPTS();
    
    PWM_1_Start();
    PWM_1_WriteCompare( 0 );                /* Red LED off */
    
    PWM_2_Start();
    PWM_2_WriteCompare( 0 );                /* Green LED off */
    
    PWM_3_Start();
    PWM_3_WriteCompare( 0 );                /* Blue LED off*/
    
    PWM_3_WritePeriod( PWM_PERIOD );        /* Set period for blue LED (PWM_3) */
    
	while( 1 )
    {
        /* Change the compare value so LED is on or off */
        PWM_3_WriteCompare( PWM_PERIOD );
        CyDelay( 100 );                 /* 100ms on (instruction counting) */
        
        PWM_3_WriteCompare( 0 );  
        CyDelay( 100 );                 /* 100 ms off */      
    }
}

/*
 * Function:	vApplicationMallocFailedHook
 *
 * Called if a malloc from the FreeRTOS heap fails. Flashes the red LED.
 * This may require more heap (configTOTAL_HEAP_SIZE in FreeRTOSVConfig.h)
 * or a different heap algorithm (heap_?.c in Source\portable\MemMang).
 *
 * Requires configUSE_MALLOC_FAILED_HOOK to be set in FreeRTOSConfig.h.
 *
 * Globals:		None
 *
 * Parameters:	Task handle and name
 *
 * Return:		Never returns - this is a fatal error condition
 */
void vApplicationMallocFailedHook( void )
{
	/* The heap space has been exceeded. */
	taskDISABLE_INTERRUPTS();
    
    PWM_1_Start();
    PWM_1_WriteCompare( 0 );                /* Red LED off */
    
    PWM_2_Start();
    PWM_2_WriteCompare( 0 );                /* Green LED off */
    
    PWM_3_Start();
    PWM_3_WriteCompare( 0 );                /* Blue LED off*/
    
    PWM_1_WritePeriod( PWM_PERIOD );                /* Set period for red LED (PWM_1) */
    
	while( 1 )
    {
        /* Change the compare value so LED is on or off */
        PWM_1_WriteCompare( PWM_PERIOD );
        CyDelay( 100 );                 /* 100ms on (instruction counting) */
        
        PWM_1_WriteCompare( 0 );  
        CyDelay( 100 );                 /* 100 ms off */      
    }
}
